@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2023, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -16,17 +16,19 @@
 #include "coefficient.hpp"
 #include "bilininteg.hpp"
 #include <random>
+#include "integrator.hpp"
+
+#include "kernel_dispatch.hpp"
 
 namespace mfem
 {
 
 /// Abstract base class LinearFormIntegrator
-class LinearFormIntegrator
+class LinearFormIntegrator : public Integrator
 {
 protected:
-   const IntegrationRule *IntRule;
 
-   LinearFormIntegrator(const IntegrationRule *ir = NULL) { IntRule = ir; }
+   LinearFormIntegrator(const IntegrationRule *ir = NULL) : Integrator(ir) {}
 
 public:
 
@@ -51,9 +53,6 @@ public:
                                        FaceElementTransformations &Tr,
                                        Vector &elvect);
 
-   virtual void SetIntRule(const IntegrationRule *ir) { IntRule = ir; }
-   const IntegrationRule* GetIntRule() { return IntRule; }
-
    virtual ~LinearFormIntegrator() { }
 };
 
@@ -104,7 +103,7 @@ public:
 };
 
 
-/// Class for domain integration L(v) := (f, v)
+/// Class for domain integration $ L(v) := (f, v) $
 class DomainLFIntegrator : public DeltaLFIntegrator
 {
    Vector shape;
@@ -112,36 +111,50 @@ class DomainLFIntegrator : public DeltaLFIntegrator
    int oa, ob;
 public:
    /// Constructs a domain integrator with a given Coefficient
-   DomainLFIntegrator(Coefficient &QF, int a = 2, int b = 0)
-   // the old default was a = 1, b = 1
-   // for simple elliptic problems a = 2, b = -2 is OK
-      : DeltaLFIntegrator(QF), Q(QF), oa(a), ob(b) { }
+   /// the old default was a = 1, b = 1
+   /// for simple elliptic problems a = 2, b = -2 is OK
+   DomainLFIntegrator(Coefficient &QF, int a = 2, int b = 0);
 
    /// Constructs a domain integrator with a given Coefficient
-   DomainLFIntegrator(Coefficient &QF, const IntegrationRule *ir)
-      : DeltaLFIntegrator(QF, ir), Q(QF), oa(1), ob(1) { }
+   DomainLFIntegrator(Coefficient &QF, const IntegrationRule *ir);
 
-   virtual bool SupportsDevice() const { return true; }
+   bool SupportsDevice() const override { return true; }
 
    /// Method defining assembly on device
-   virtual void AssembleDevice(const FiniteElementSpace &fes,
-                               const Array<int> &markers,
-                               Vector &b);
+   void AssembleDevice(const FiniteElementSpace &fes,
+                       const Array<int> &markers,
+                       Vector &b) override;
 
    /** Given a particular Finite Element and a transformation (Tr)
        computes the element right hand side element vector, elvect. */
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void AssembleDeltaElementVect(const FiniteElement &fe,
-                                         ElementTransformation &Trans,
-                                         Vector &elvect);
+   void AssembleDeltaElementVect(const FiniteElement &fe,
+                                 ElementTransformation &Trans,
+                                 Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
+
+   /// args: vdim, ne, d1d, q1d, map_type, markers, B, detJ, W, coeff, y
+   using AssembleKernelType = void (*)(const int, const int, const int,
+                                       const int, const int, const int *,
+                                       const real_t *, const real_t *,
+                                       const real_t *, const Vector &coeff,
+                                       real_t *y);
+
+   /// parameters: use DIM, T_D1D, T_Q1D
+   MFEM_REGISTER_KERNELS(AssembleKernels, AssembleKernelType, (int, int, int));
+   struct Kernels { Kernels(); };
+
+   template <int DIM, int D1D, int Q1D> static void AddSpecialization()
+   {
+      AssembleKernels::Specialization<DIM, D1D, Q1D>::Add();
+   }
 };
 
-/// Class for domain integrator L(v) := (f, grad v)
+/// Class for domain integrator $ L(v) := (f, \nabla v) $
 class DomainLFGradIntegrator : public DeltaLFIntegrator
 {
 private:
@@ -150,32 +163,32 @@ private:
    DenseMatrix dshape;
 
 public:
-   /// Constructs the domain integrator (Q, grad v)
+   /// Constructs the domain integrator $ (Q, \nabla v) $
    DomainLFGradIntegrator(VectorCoefficient &QF)
       : DeltaLFIntegrator(QF), Q(QF) { }
 
-   virtual bool SupportsDevice() const { return true; }
+   bool SupportsDevice() const override { return true; }
 
    /// Method defining assembly on device
-   virtual void AssembleDevice(const FiniteElementSpace &fes,
-                               const Array<int> &markers,
-                               Vector &b);
+   void AssembleDevice(const FiniteElementSpace &fes,
+                       const Array<int> &markers,
+                       Vector &b) override;
 
    /** Given a particular Finite Element and a transformation (Tr)
        computes the element right hand side element vector, elvect. */
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void AssembleDeltaElementVect(const FiniteElement &fe,
-                                         ElementTransformation &Trans,
-                                         Vector &elvect);
+   void AssembleDeltaElementVect(const FiniteElement &fe,
+                                 ElementTransformation &Trans,
+                                 Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
 
-/// Class for boundary integration L(v) := (g, v)
+/// Class for boundary integration $ L(v) := (g, v) $
 class BoundaryLFIntegrator : public LinearFormIntegrator
 {
    Vector shape;
@@ -187,26 +200,26 @@ public:
    BoundaryLFIntegrator(Coefficient &QG, int a = 1, int b = 1)
       : Q(QG), oa(a), ob(b) { }
 
-   virtual bool SupportsDevice() const { return true; }
+   bool SupportsDevice() const override { return true; }
 
    /// Method defining assembly on device
-   virtual void AssembleDevice(const FiniteElementSpace &fes,
-                               const Array<int> &markers,
-                               Vector &b);
+   void AssembleDevice(const FiniteElementSpace &fes,
+                       const Array<int> &markers,
+                       Vector &b) override;
 
    /** Given a particular boundary Finite Element and a transformation (Tr)
        computes the element boundary vector, elvect. */
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       FaceElementTransformations &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               FaceElementTransformations &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/// Class for boundary integration \f$ L(v) = (g \cdot n, v) \f$
+/// Class for boundary integration $ L(v) = (g \cdot n, v) $
 class BoundaryNormalLFIntegrator : public LinearFormIntegrator
 {
    Vector shape;
@@ -217,21 +230,21 @@ public:
    BoundaryNormalLFIntegrator(VectorCoefficient &QG, int a = 1, int b = 1)
       : Q(QG), oa(a), ob(b) { }
 
-   virtual bool SupportsDevice() const { return true; }
+   bool SupportsDevice() const override { return true; }
 
    /// Method defining assembly on device
-   virtual void AssembleDevice(const FiniteElementSpace &fes,
-                               const Array<int> &markers,
-                               Vector &b);
+   void AssembleDevice(const FiniteElementSpace &fes,
+                       const Array<int> &markers,
+                       Vector &b) override;
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/// Class for boundary integration \f$ L(v) = (g \cdot \tau, v) \f$ in 2D
+/// Class for boundary integration $ L(v) = (g \cdot \tau, v) $ in 2D
 class BoundaryTangentialLFIntegrator : public LinearFormIntegrator
 {
    Vector shape;
@@ -242,15 +255,15 @@ public:
    BoundaryTangentialLFIntegrator(VectorCoefficient &QG, int a = 1, int b = 1)
       : Q(QG), oa(a), ob(b) { }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/** Class for domain integration of L(v) := (f, v), where
-    f=(f1,...,fn) and v=(v1,...,vn). */
+/** Class for domain integration of $ L(v) := (f, v) $, where
+    $ f = (f_1,\dots,f_n)$ and $ v = (v_1,\dots,v_n) $. */
 class VectorDomainLFIntegrator : public DeltaLFIntegrator
 {
 private:
@@ -259,31 +272,36 @@ private:
 
 public:
    /// Constructs a domain integrator with a given VectorCoefficient
-   VectorDomainLFIntegrator(VectorCoefficient &QF)
-      : DeltaLFIntegrator(QF), Q(QF) { }
+   VectorDomainLFIntegrator(VectorCoefficient &QF,
+                            const IntegrationRule *ir = nullptr);
 
-   virtual bool SupportsDevice() const { return true; }
+   bool SupportsDevice() const override { return true; }
 
    /// Method defining assembly on device
-   virtual void AssembleDevice(const FiniteElementSpace &fes,
-                               const Array<int> &markers,
-                               Vector &b);
+   void AssembleDevice(const FiniteElementSpace &fes, const Array<int> &markers,
+                       Vector &b) override;
 
    /** Given a particular Finite Element and a transformation (Tr)
        computes the element right hand side element vector, elvect. */
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void AssembleDeltaElementVect(const FiniteElement &fe,
-                                         ElementTransformation &Trans,
-                                         Vector &elvect);
+   void AssembleDeltaElementVect(const FiniteElement &fe,
+                                 ElementTransformation &Trans,
+                                 Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
+
+   template <int DIM, int D1D, int Q1D> static void AddSpecialization()
+   {
+      // uses the same kernels for assembly
+      DomainLFIntegrator::AssembleKernels::Specialization<DIM, D1D, Q1D>::Add();
+   }
 };
 
-/** Class for domain integrator L(v) := (f, grad v), where
-    f=(f1x,f1y,f1z,...,fnx,fny,fnz) and v=(v1,...,vn). */
+/** Class for domain integrator $ L(v) := (f, \nabla v) $, where
+    $ f = (f_{1x},f_{1y},f_{1z},\dots,f_{nx},f_{ny},f_{nz})$ and $v=(v_1,\dots,v_n)$. */
 class VectorDomainLFGradIntegrator : public DeltaLFIntegrator
 {
 private:
@@ -296,28 +314,28 @@ public:
    VectorDomainLFGradIntegrator(VectorCoefficient &QF)
       : DeltaLFIntegrator(QF), Q(QF) { }
 
-   virtual bool SupportsDevice() const override { return true; }
+   bool SupportsDevice() const override { return true; }
 
    /// Method defining assembly on device
-   virtual void AssembleDevice(const FiniteElementSpace &fes,
-                               const Array<int> &markers,
-                               Vector &b) override;
+   void AssembleDevice(const FiniteElementSpace &fes,
+                       const Array<int> &markers,
+                       Vector &b) override;
 
    /** Given a particular Finite Element and a transformation (Tr)
        computes the element right hand side element vector, elvect. */
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect) override;
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void AssembleDeltaElementVect(const FiniteElement &fe,
-                                         ElementTransformation &Trans,
-                                         Vector &elvect) override;
+   void AssembleDeltaElementVect(const FiniteElement &fe,
+                                 ElementTransformation &Trans,
+                                 Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/** Class for boundary integration of L(v) := (g, v), where
-    f=(f1,...,fn) and v=(v1,...,vn). */
+/** Class for boundary integration of $ L(v) := (g, v) $, where
+    $g=(g_1,\dots,g_n)$ and $v=(v_1,\dots,v_n)$. */
 class VectorBoundaryLFIntegrator : public LinearFormIntegrator
 {
 private:
@@ -330,19 +348,19 @@ public:
 
    /** Given a particular boundary Finite Element and a transformation (Tr)
        computes the element boundary vector, elvect. */
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
    // For DG spaces
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       FaceElementTransformations &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               FaceElementTransformations &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/// \f$ (f, v)_{\Omega} \f$ for VectorFiniteElements (Nedelec, Raviart-Thomas)
+/// $ (f, v)_{\Omega} $ for VectorFiniteElements (Nedelec, Raviart-Thomas)
 class VectorFEDomainLFIntegrator : public DeltaLFIntegrator
 {
 private:
@@ -354,24 +372,24 @@ public:
    VectorFEDomainLFIntegrator(VectorCoefficient &F)
       : DeltaLFIntegrator(F), QF(F) { }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void AssembleDeltaElementVect(const FiniteElement &fe,
-                                         ElementTransformation &Trans,
-                                         Vector &elvect);
+   void AssembleDeltaElementVect(const FiniteElement &fe,
+                                 ElementTransformation &Trans,
+                                 Vector &elvect) override;
 
-   virtual bool SupportsDevice() const { return true; }
+   bool SupportsDevice() const override { return true; }
 
-   virtual void AssembleDevice(const FiniteElementSpace &fes,
-                               const Array<int> &markers,
-                               Vector &b);
+   void AssembleDevice(const FiniteElementSpace &fes,
+                       const Array<int> &markers,
+                       Vector &b) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/// \f$ (Q, curl v)_{\Omega} \f$ for Nedelec Elements)
+/// $ (Q, \mathrm{curl}(v))_{\Omega} $ for Nedelec Elements
 class VectorFEDomainLFCurlIntegrator : public DeltaLFIntegrator
 {
 private:
@@ -380,69 +398,69 @@ private:
    Vector vec;
 
 public:
-   /// Constructs the domain integrator (Q, curl v)
+   /// Constructs the domain integrator $(Q, \mathrm{curl}(v))  $
    VectorFEDomainLFCurlIntegrator(VectorCoefficient &F)
       : DeltaLFIntegrator(F), QF(&F) { }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void AssembleDeltaElementVect(const FiniteElement &fe,
-                                         ElementTransformation &Trans,
-                                         Vector &elvect);
+   void AssembleDeltaElementVect(const FiniteElement &fe,
+                                 ElementTransformation &Trans,
+                                 Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/// \f$ (Q, div v)_{\Omega} \f$ for RT Elements)
+/// $ (Q, \mathrm{div}(v))_{\Omega} $ for RT Elements
 class VectorFEDomainLFDivIntegrator : public DeltaLFIntegrator
 {
 private:
    Vector divshape;
    Coefficient &Q;
 public:
-   /// Constructs the domain integrator (Q, div v)
+   /// Constructs the domain integrator $ (Q, \mathrm{div}(v)) $
    VectorFEDomainLFDivIntegrator(Coefficient &QF)
       : DeltaLFIntegrator(QF), Q(QF) { }
 
    /** Given a particular Finite Element and a transformation (Tr)
        computes the element right hand side element vector, elvect. */
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void AssembleDeltaElementVect(const FiniteElement &fe,
-                                         ElementTransformation &Trans,
-                                         Vector &elvect);
+   void AssembleDeltaElementVect(const FiniteElement &fe,
+                                 ElementTransformation &Trans,
+                                 Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/** \f$ (f, v \cdot n)_{\partial\Omega} \f$ for vector test function
-    v=(v1,...,vn) where all vi are in the same scalar FE space and f is a
+/** $ (f, v \cdot n)_{\partial\Omega} $ for vector test function
+    $v=(v_1,\dots,v_n)$ where all vi are in the same scalar FE space and $f$ is a
     scalar function. */
 class VectorBoundaryFluxLFIntegrator : public LinearFormIntegrator
 {
 private:
-   double Sign;
+   real_t Sign;
    Coefficient *F;
    Vector shape, nor;
 
 public:
-   VectorBoundaryFluxLFIntegrator(Coefficient &f, double s = 1.0,
+   VectorBoundaryFluxLFIntegrator(Coefficient &f, real_t s = 1.0,
                                   const IntegrationRule *ir = NULL)
       : LinearFormIntegrator(ir), Sign(s), F(&f) { }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
-/** Class for boundary integration of (f, v.n) for scalar coefficient f and
-    RT vector test function v. This integrator works with RT spaces defined
+/** Class for boundary integration of $ (f, v \cdot n) $ for scalar coefficient $f$ and
+    RT vector test function $v$. This integrator works with RT spaces defined
     using the RT_FECollection class. */
 class VectorFEBoundaryFluxLFIntegrator : public LinearFormIntegrator
 {
@@ -457,20 +475,39 @@ public:
    VectorFEBoundaryFluxLFIntegrator(Coefficient &f, int a = 2, int b = 0)
       : F(&f), oa(a), ob(b) { }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 
-   virtual bool SupportsDevice() const { return true; }
+   bool SupportsDevice() const override { return true; }
 
-   virtual void AssembleDevice(const FiniteElementSpace &fes,
-                               const Array<int> &markers,
-                               Vector &b);
+   void AssembleDevice(const FiniteElementSpace &fes,
+                       const Array<int> &markers,
+                       Vector &b) override;
 };
 
-/// Class for boundary integration \f$ L(v) = (n \times f, v) \f$
+/** Class for boundary integration of (f.n, v.n) for vector coefficient f and
+    RT vector test function v. This integrator works with RT spaces defined
+    using the RT_FECollection class. */
+class VectorFEBoundaryNormalLFIntegrator : public LinearFormIntegrator
+{
+private:
+   VectorCoefficient &F;
+   Vector shape;
+
+public:
+   VectorFEBoundaryNormalLFIntegrator(VectorCoefficient &f) : F(f) { }
+
+   virtual void AssembleRHSElementVect(const FiniteElement &el,
+                                       ElementTransformation &Tr,
+                                       Vector &elvect) override;
+
+   using LinearFormIntegrator::AssembleRHSElementVect;
+};
+
+/// Class for boundary integration $ L(v) = (n \times f, v) $
 class VectorFEBoundaryTangentLFIntegrator : public LinearFormIntegrator
 {
 private:
@@ -482,42 +519,42 @@ public:
                                        int a = 2, int b = 0)
       : f(QG), oa(a), ob(b) { }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
 
 
 /** Class for boundary integration of the linear form:
-    (alpha/2) < (u.n) f, w > - beta < |u.n| f, w >,
-    where f and u are given scalar and vector coefficients, respectively,
-    and w is the scalar test function. */
+    $ \frac{\alpha}{2} \langle (u \cdot n) f, w \rangle - \beta \langle |u \cdot n| f, w \rangle $
+    where $f$ and $u$ are given scalar and vector coefficients, respectively,
+    and $w$ is the scalar test function. */
 class BoundaryFlowIntegrator : public LinearFormIntegrator
 {
 private:
    Coefficient *f;
    VectorCoefficient *u;
-   double alpha, beta;
+   real_t alpha, beta;
 
    Vector shape;
 
 public:
    BoundaryFlowIntegrator(Coefficient &f_, VectorCoefficient &u_,
-                          double a)
+                          real_t a)
    { f = &f_; u = &u_; alpha = a; beta = 0.5*a; }
 
    BoundaryFlowIntegrator(Coefficient &f_, VectorCoefficient &u_,
-                          double a, double b)
+                          real_t a, real_t b)
    { f = &f_; u = &u_; alpha = a; beta = b; }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       FaceElementTransformations &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               FaceElementTransformations &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
@@ -525,41 +562,41 @@ public:
 
 /** Boundary linear integrator for imposing non-zero Dirichlet boundary
     conditions, to be used in conjunction with DGDiffusionIntegrator.
-    Specifically, given the Dirichlet data u_D, the linear form assembles the
+    Specifically, given the Dirichlet data $u_D$, the linear form assembles the
     following integrals on the boundary:
-
-    sigma < u_D, (Q grad(v)).n > + kappa < {h^{-1} Q} u_D, v >,
-
+   $$
+    \sigma \langle u_D, (Q \nabla v) \cdot n \rangle + \kappa \langle {h^{-1} Q} u_D, v \rangle,
+   $$
     where Q is a scalar or matrix diffusion coefficient and v is the test
-    function. The parameters sigma and kappa should be the same as the ones
+    function. The parameters $\sigma$ and $\kappa$ should be the same as the ones
     used in the DGDiffusionIntegrator. */
 class DGDirichletLFIntegrator : public LinearFormIntegrator
 {
 protected:
    Coefficient *uD, *Q;
    MatrixCoefficient *MQ;
-   double sigma, kappa;
+   real_t sigma, kappa;
 
    // these are not thread-safe!
    Vector shape, dshape_dn, nor, nh, ni;
    DenseMatrix dshape, mq, adjJ;
 
 public:
-   DGDirichletLFIntegrator(Coefficient &u, const double s, const double k)
+   DGDirichletLFIntegrator(Coefficient &u, const real_t s, const real_t k)
       : uD(&u), Q(NULL), MQ(NULL), sigma(s), kappa(k) { }
    DGDirichletLFIntegrator(Coefficient &u, Coefficient &q,
-                           const double s, const double k)
+                           const real_t s, const real_t k)
       : uD(&u), Q(&q), MQ(NULL), sigma(s), kappa(k) { }
    DGDirichletLFIntegrator(Coefficient &u, MatrixCoefficient &q,
-                           const double s, const double k)
+                           const real_t s, const real_t k)
       : uD(&u), Q(NULL), MQ(&q), sigma(s), kappa(k) { }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       FaceElementTransformations &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               FaceElementTransformations &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
@@ -568,19 +605,19 @@ public:
 /** Boundary linear form integrator for imposing non-zero Dirichlet boundary
     conditions, in a DG elasticity formulation. Specifically, the linear form is
     given by
-
-    alpha < u_D, (lambda div(v) I + mu (grad(v) + grad(v)^T)) . n > +
-      + kappa < h^{-1} (lambda + 2 mu) u_D, v >,
-
-    where u_D is the given Dirichlet data. The parameters alpha, kappa, lambda
-    and mu, should match the parameters with the same names used in the bilinear
+   $$
+    \alpha \langle u_D, (\lambda \mathrm{div}(v) I + \mu (\nabla v + \nabla v^{\mathrm{T}})) \cdot n \rangle +
+      + \kappa \langle h^{-1} (\lambda + 2 \mu) u_D, v \rangle,
+   $$
+    where u_D is the given Dirichlet data. The parameters $\alpha$, $\kappa$, $\lambda$
+    and $\mu$, should match the parameters with the same names used in the bilinear
     form integrator, DGElasticityIntegrator. */
 class DGElasticityDirichletLFIntegrator : public LinearFormIntegrator
 {
 protected:
    VectorCoefficient &uD;
    Coefficient *lambda, *mu;
-   double alpha, kappa;
+   real_t alpha, kappa;
 
 #ifndef MFEM_THREAD_SAFE
    Vector shape;
@@ -596,15 +633,15 @@ protected:
 public:
    DGElasticityDirichletLFIntegrator(VectorCoefficient &uD_,
                                      Coefficient &lambda_, Coefficient &mu_,
-                                     double alpha_, double kappa_)
+                                     real_t alpha_, real_t kappa_)
       : uD(uD_), lambda(&lambda_), mu(&mu_), alpha(alpha_), kappa(kappa_) { }
 
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       FaceElementTransformations &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               FaceElementTransformations &Tr,
+                               Vector &elvect) override;
 
    using LinearFormIntegrator::AssembleRHSElementVect;
 };
@@ -612,18 +649,18 @@ public:
 
 /** Class for spatial white Gaussian noise integration.
 
-    The target problem is the linear SPDE a(u,v) = F(v) with F(v) := <Ẇ,v>,
-    where Ẇ is spatial white Gaussian noise. When the Galerkin method is used to
-    discretize this problem into a linear system of equations Ax = b, the RHS is
-    a Gaussian random vector b~N(0,M) whose covariance matrix is the same as the
-    mass matrix M_ij = (v_i,v_j). This property can be ensured if b = H w, where
-    HHᵀ = M and each component w_i~N(0,1).
+    The target problem is the linear SPDE $ a(u,v) = F(v)$ with $F(v) := <\dot{W},v> $,
+    where $\dot{W}$ is spatial white Gaussian noise. When the Galerkin method is used to
+    discretize this problem into a linear system of equations $Ax = b$, the RHS is
+    a Gaussian random vector $b \sim N(0,M)$ whose covariance matrix is the same as the
+    mass matrix $M_{ij} = (v_i,v_j)$. This property can be ensured if $b = H w$, where
+    $HH^{\mathrm{T}} = M$ and each component $w_i\sim N(0,1)$.
 
-    There is much flexibility in how we may wish to define H. In this PR, we
-    define H = Pᵀ diag(L_e), where P is the local-to-global dof assembly matrix
-    and diag(L_e) is a block-diagonal matrix with L_e L_eᵀ = M_e, where M_e is
-    the element mass matrix for element e. A straightforward computation shows
-    that HHᵀ = Pᵀ diag(M_e) P = M, as necessary. */
+    There is much flexibility in how we may wish to define $H$. In this PR, we
+    define $H = P^{\mathrm{T}} diag(L_e)$, where $P$ is the local-to-global dof assembly matrix
+    and $\mathrm{diag}(L_e)$ is a block-diagonal matrix with $L_e L_e^{\mathrm{T}} = M_e$, where $M_e$ is
+    the element mass matrix for element $e$. A straightforward computation shows
+    that $HH^{\mathrm{T}} = P^{\mathrm{T}} diag(M_e) P = M$, as necessary. */
 class WhiteGaussianNoiseDomainLFIntegrator : public LinearFormIntegrator
 {
 #ifdef MFEM_USE_MPI
@@ -634,7 +671,7 @@ class WhiteGaussianNoiseDomainLFIntegrator : public LinearFormIntegrator
 
    // Define random generator with Gaussian distribution
    std::default_random_engine generator;
-   std::normal_distribution<double> dist;
+   std::normal_distribution<real_t> dist;
 
    bool save_factors = false;
 public:
@@ -657,7 +694,7 @@ public:
       int myid;
       MPI_Comm_rank(comm, &myid);
 
-      int seed = (seed_ > 0) ? seed_ + myid : time(0) + myid;
+      int seed = (seed_ > 0) ? seed_ + myid : time(nullptr) + myid;
       SetSeed(seed);
    }
 #else
@@ -676,9 +713,9 @@ public:
    }
 
    using LinearFormIntegrator::AssembleRHSElementVect;
-   virtual void AssembleRHSElementVect(const FiniteElement &el,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &el,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
    /** @brief Saves the lower triangular matrices in the element-wise Cholesky
        decomposition. The parameter @a NE should be the number of elements in
@@ -718,8 +755,8 @@ public:
 };
 
 
-/** Class for domain integration of L(v) := (f, v), where
-    f=(f1,...,fn) and v=(v1,...,vn). that makes use of
+/** Class for domain integration of $ L(v) := (f, v) $, where
+    $ f=(f_1,\dots,f_n)$ and $v=(v_1,\dots,v_n)$. that makes use of
     VectorQuadratureFunctionCoefficient*/
 class VectorQuadratureLFIntegrator : public LinearFormIntegrator
 {
@@ -728,7 +765,7 @@ private:
 
 public:
    VectorQuadratureLFIntegrator(VectorQuadratureFunctionCoefficient &vqfc,
-                                const IntegrationRule *ir)
+                                const IntegrationRule *ir = NULL)
       : LinearFormIntegrator(ir), vqfc(vqfc)
    {
       if (ir)
@@ -739,11 +776,11 @@ public:
    }
 
    using LinearFormIntegrator::AssembleRHSElementVect;
-   virtual void AssembleRHSElementVect(const FiniteElement &fe,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &fe,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void SetIntRule(const IntegrationRule *ir)
+   void SetIntRule(const IntegrationRule *ir) override
    {
       MFEM_WARNING("Integration rule not used in this class. "
                    "The QuadratureFunction integration rules are used instead");
@@ -751,7 +788,7 @@ public:
 };
 
 
-/** Class for domain integration L(v) := (f, v) that makes use
+/** Class for domain integration $ L(v) := (f, v) $ that makes use
     of QuadratureFunctionCoefficient. */
 class QuadratureLFIntegrator : public LinearFormIntegrator
 {
@@ -760,7 +797,7 @@ private:
 
 public:
    QuadratureLFIntegrator(QuadratureFunctionCoefficient &qfc,
-                          const IntegrationRule *ir)
+                          const IntegrationRule *ir = NULL)
       : LinearFormIntegrator(ir), qfc(qfc)
    {
       if (ir)
@@ -771,11 +808,11 @@ public:
    }
 
    using LinearFormIntegrator::AssembleRHSElementVect;
-   virtual void AssembleRHSElementVect(const FiniteElement &fe,
-                                       ElementTransformation &Tr,
-                                       Vector &elvect);
+   void AssembleRHSElementVect(const FiniteElement &fe,
+                               ElementTransformation &Tr,
+                               Vector &elvect) override;
 
-   virtual void SetIntRule(const IntegrationRule *ir)
+   void SetIntRule(const IntegrationRule *ir) override
    {
       MFEM_WARNING("Integration rule not used in this class. "
                    "The QuadratureFunction integration rules are used instead");
