@@ -21,6 +21,11 @@
 #define rmdir(dir) _rmdir(dir)
 #endif
 
+#ifdef MFEM_USE_HDF5
+#include <hdf5.h>
+#include <hdf5_hl.h>
+#endif
+
 using namespace mfem;
 
 TEST_CASE("Save and load from collections", "[DataCollection]")
@@ -325,3 +330,55 @@ TEST_CASE("ParaView restart mode", "[ParaView]")
    REQUIRE(remove("ParaView/ParaView.pvd") == 0);
    REQUIRE(rmdir("ParaView") == 0);
 }
+
+#ifdef MFEM_USE_HDF5
+
+TEST_CASE("ParaView VTKHDF restart mode", "[ParaView]")
+{
+   Mesh mesh = Mesh::MakeCartesian2D(2, 3, Element::QUADRILATERAL);
+   H1_FECollection fec(1, mesh.Dimension());
+   FiniteElementSpace fes(&mesh, &fec);
+   GridFunction u(&fes);
+   u = 0.0;
+
+   // Write initial dataset with three timesteps: 0, 1, 2.
+   {
+      ParaViewHDFDataCollection dc("ParaView", &mesh);
+      dc.RegisterField("u", &u);
+      SaveDataCollection(dc, 0, 0);
+      SaveDataCollection(dc, 1, 1);
+      SaveDataCollection(dc, 2, 2);
+   }
+
+   auto read_time_vals = []()
+   {
+      const hid_t f = H5Fopen("ParaView.vtkhdf", H5F_ACC_RDONLY, H5P_DEFAULT);
+      hsize_t size;
+      H5LTget_dataset_info(f, "/VTKHDF/Steps/Values", &size, nullptr, nullptr);
+      std::vector<double> vals(size);
+      H5LTread_dataset(f, "/VTKHDF/Steps/Values", H5T_NATIVE_DOUBLE, vals.data());
+      H5Fclose(f);
+      return vals;
+   };
+
+   // Check that the time steps were written correctly
+   REQUIRE(read_time_vals() == std::vector<double>({0.0, 1.0, 2.0}));
+
+   // Using restart mode, append to the existing dataset, overwriting timesteps
+   // 1 and 2 with 1 and 1.5.
+   {
+      ParaViewHDFDataCollection dc("ParaView", &mesh);
+      dc.UseRestartMode(true);
+      dc.RegisterField("u", &u);
+      SaveDataCollection(dc, 1, 1.0);
+      SaveDataCollection(dc, 2, 1.5);
+   }
+
+   // Check that the time steps were updated correctly
+   REQUIRE(read_time_vals() == std::vector<double>({0.0, 1.0, 1.5}));
+
+   // Clean up
+   REQUIRE(remove("ParaView.vtkhdf") == 0);
+}
+
+#endif // MFEM_USE_HDF5
