@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -9,12 +9,14 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
+#ifdef _WIN32
+#define _USE_MATH_DEFINES
+#include <cmath>
+#endif
+
 #include "unit_tests.hpp"
 #include "mfem.hpp"
 
-#include <fstream>
-#include <iostream>
-
 using namespace mfem;
 
 namespace pa_kernels
@@ -188,7 +190,7 @@ void pa_divergence_transpose_testnd(int dim)
    pa_mixed_transpose_test<VectorDivergenceIntegrator>(fes1, fes2);
 }
 
-TEST_CASE("PA VectorDivergence", "[PartialAssembly], [CUDA]")
+TEST_CASE("PA VectorDivergence", "[PartialAssembly], [GPU]")
 {
    SECTION("2D")
    {
@@ -280,7 +282,7 @@ void pa_gradient_transpose_testnd(int dim, FECType fec_type)
    pa_mixed_transpose_test<GradientIntegrator>(fes1, fes2);
 }
 
-TEST_CASE("PA Gradient", "[PartialAssembly], [CUDA]")
+TEST_CASE("PA Gradient", "[PartialAssembly], [GPU]")
 {
    auto fec_type = GENERATE(FECType::H1, FECType::L2_VALUE,
                             FECType::L2_INTEGRAL);
@@ -329,7 +331,7 @@ real_t test_nl_convection_nd(int dim)
    return difference;
 }
 
-TEST_CASE("Nonlinear Convection", "[PartialAssembly], [NonlinearPA], [CUDA]")
+TEST_CASE("Nonlinear Convection", "[PartialAssembly], [NonlinearPA], [GPU]")
 {
    SECTION("2D")
    {
@@ -343,60 +345,129 @@ TEST_CASE("Nonlinear Convection", "[PartialAssembly], [NonlinearPA], [CUDA]")
 }
 
 template <typename INTEGRATOR>
-real_t test_vector_pa_integrator(int dim)
+real_t test_pa_vector_integrator(int dim, int sdim)
 {
-   Mesh mesh = MakeCartesianNonaligned(dim, 2);
-   int order = 2;
-   H1_FECollection fec(order, dim);
-   FiniteElementSpace fes(&mesh, &fec, dim);
+   const bool all = launch_all_non_regression_tests;
+   const auto NE = all ? GENERATE(1, 2, 3) : 2;
+   const auto p = all ? GENERATE(1, 2, 3): 2;
+   CAPTURE(p, NE);
+
+   Mesh mesh = MakeCartesianNonaligned(dim, NE);
+   mesh.SetCurvature(p, false, sdim);
+
+   H1_FECollection fec(p, dim);
+   FiniteElementSpace fes(&mesh, &fec, sdim);
 
    GridFunction x(&fes), y_fa(&fes), y_pa(&fes);
    x.Randomize(1);
 
+   ConstantCoefficient const_coeff(M_PI_2);
+   FunctionCoefficient funct_coeff([](const Vector &x) { return M_1_PI + x[0]*x[0]; });
+
+   Vector val(dim); val = 1.0;
+   VectorConstantCoefficient v_const_coeff(val);
+   VectorFunctionCoefficient v_funct_coeff(dim, [&](const Vector &x, Vector &v)
+   {
+      v(0) = M_LN2 * x(0);
+      if (dim > 1) { v(1) = M_E * x(1); }
+      if (dim > 2) { v(2) = M_PI * x(2); }
+   });
+
+   MatrixFunctionCoefficient m_funct_coeff(dim, [&](const Vector &x,
+                                                    DenseMatrix &f)
+   {
+      f = 0.0;
+      if (dim == 1)
+      {
+         f(0,0) = 1.1 + sin(M_PI * x[0]);  // 1,1
+      }
+      else if (dim == 2)
+      {
+         f(0,0) = 1.1 + sin(M_PI * x[1]);  // 1,1
+         f(1,0) = cos(1.3 * M_PI * x[1]);  // 2,1
+         f(0,1) = cos(2.5 * M_PI * x[0]);  // 1,2
+         f(1,1) = 1.1 + sin(4.9 * M_PI * x[0]);  // 2,2
+      }
+      else if (dim == 3)
+      {
+         f(0,0) = 1.1 + sin(M_PI * x[1]);  // 1,1
+         f(0,1) = cos(2.5 * M_PI * x[0]);  // 1,2
+         f(0,2) = sin(4.9 * M_PI * x[2]);  // 1,3
+         f(1,0) = cos(M_PI * x[0]);  // 2,1
+         f(1,1) = 1.1 + sin(6.1 * M_PI * x[1]);  // 2,2
+         f(1,2) = cos(6.1 * M_PI * x[2]);  // 2,3
+         f(2,0) = sin(1.5 * M_PI * x[1]);  // 3,1
+         f(2,1) = cos(2.9 * M_PI * x[0]);  // 3,2
+         f(2,2) = 1.1 + sin(6.1 * M_PI * x[2]);  // 3,3
+      }
+   });
+   REQUIRE((sdim > dim || m_funct_coeff.GetVDim() == fes.GetVDim()));
+
    BilinearForm blf_fa(&fes);
+   blf_fa.SetAssemblyLevel(AssemblyLevel::LEGACY);
+   // scalar coefficients
    blf_fa.AddDomainIntegrator(new INTEGRATOR);
+   blf_fa.AddDomainIntegrator(new INTEGRATOR(const_coeff));
+   blf_fa.AddDomainIntegrator(new INTEGRATOR(funct_coeff));
+   if (sdim == dim)
+   {
+      // vector coefficients
+      blf_fa.AddDomainIntegrator(new INTEGRATOR(v_const_coeff));
+      blf_fa.AddDomainIntegrator(new INTEGRATOR(v_funct_coeff));
+      // matrix coefficients
+      blf_fa.AddDomainIntegrator(new INTEGRATOR(m_funct_coeff));
+   }
    blf_fa.Assemble();
    blf_fa.Finalize();
    blf_fa.Mult(x, y_fa);
 
    BilinearForm blf_pa(&fes);
    blf_pa.SetAssemblyLevel(AssemblyLevel::PARTIAL);
+   // scalar coefficients
    blf_pa.AddDomainIntegrator(new INTEGRATOR);
+   blf_pa.AddDomainIntegrator(new INTEGRATOR(const_coeff));
+   blf_pa.AddDomainIntegrator(new INTEGRATOR(funct_coeff));
+   if (sdim == dim)
+   {
+      // vector coefficients
+      blf_pa.AddDomainIntegrator(new INTEGRATOR(v_const_coeff));
+      blf_pa.AddDomainIntegrator(new INTEGRATOR(v_funct_coeff));
+      // matrix coefficients
+      blf_pa.AddDomainIntegrator(new INTEGRATOR(m_funct_coeff));
+   }
    blf_pa.Assemble();
    blf_pa.Mult(x, y_pa);
 
    y_fa -= y_pa;
-   real_t difference = y_fa.Norml2();
 
-   return difference;
+   return y_fa.Norml2();
 }
 
-TEST_CASE("PA Vector Mass", "[PartialAssembly], [VectorPA], [CUDA]")
+TEST_CASE("PA Vector Mass",
+          "[PartialAssembly][VectorPA][VectorMassPA][GPU]")
 {
-   SECTION("2D")
-   {
-      REQUIRE(test_vector_pa_integrator<VectorMassIntegrator>(2) == MFEM_Approx(0.0));
-   }
-
-   SECTION("3D")
-   {
-      REQUIRE(test_vector_pa_integrator<VectorMassIntegrator>(3) == MFEM_Approx(0.0));
-   }
+   const auto DIM = GENERATE(2, 3);
+   CAPTURE(DIM);
+   REQUIRE(test_pa_vector_integrator<VectorMassIntegrator>(DIM, DIM)
+           == MFEM_Approx(0.0));
 }
 
-TEST_CASE("PA Vector Diffusion", "[PartialAssembly], [VectorPA], [CUDA]")
+TEST_CASE("PA Vector Diffusion",
+          "[PartialAssembly][VectorPA][VectorDiffusionPA][GPU]")
 {
-   SECTION("2D")
-   {
-      REQUIRE(test_vector_pa_integrator<VectorDiffusionIntegrator>(2)
-              == MFEM_Approx(0.0));
-   }
+   const auto DIM = GENERATE(2, 3);
+   CAPTURE(DIM);
+   REQUIRE(test_pa_vector_integrator<VectorDiffusionIntegrator>(DIM, DIM)
+           == MFEM_Approx(0.0));
+}
 
-   SECTION("3D")
-   {
-      REQUIRE(test_vector_pa_integrator<VectorDiffusionIntegrator>(3)
-              == MFEM_Approx(0.0));
-   }
+TEST_CASE("PA Vector Diffusion 2D/3D",
+          "[PartialAssembly][VectorPA][VectorDiffusionPA][CUDA]")
+{
+   const int DIM = 2, SDIM = 3;
+   CAPTURE(DIM, SDIM);
+   REQUIRE(test_pa_vector_integrator<VectorDiffusionIntegrator>(DIM, SDIM)
+           == MFEM_Approx(0.0));
 }
 
 void velocity_function(const Vector &x, Vector &v)
@@ -507,7 +578,7 @@ void test_pa_convection(const std::string &meshname, int order, int prob,
 }
 
 // Basic unit tests for convection
-TEST_CASE("PA Convection", "[PartialAssembly], [CUDA]")
+TEST_CASE("PA Convection", "[PartialAssembly], [GPU]")
 {
    // prob:
    // - 0: CG,
@@ -533,7 +604,7 @@ TEST_CASE("PA Convection", "[PartialAssembly], [CUDA]")
 } // test case
 
 // Advanced unit tests for convection
-TEST_CASE("PA Convection advanced", "[PartialAssembly], [MFEMData], [CUDA]")
+TEST_CASE("PA Convection advanced", "[PartialAssembly], [MFEMData], [GPU]")
 {
    if (launch_all_non_regression_tests)
    {
@@ -588,7 +659,7 @@ static void test_pa_integrator()
    // Don't use a special integration rule if q_order_inc == 0
    const bool use_ir = q_order_inc > 0;
    const IntegrationRule *ir =
-      use_ir ? &IntRules.Get(mesh.GetElementGeometry(0), q_order) : nullptr;
+      use_ir ? &IntRules.Get(mesh.GetTypicalElementGeometry(), q_order) : nullptr;
 
    GridFunction x(&fes), y_fa(&fes), y_pa(&fes);
    x.Randomize(1);
@@ -612,17 +683,17 @@ static void test_pa_integrator()
    REQUIRE(y_fa.Normlinf() == MFEM_Approx(0.0));
 }
 
-TEST_CASE("PA Mass", "[PartialAssembly], [CUDA]")
+TEST_CASE("PA Mass", "[PartialAssembly], [GPU]")
 {
    test_pa_integrator<MassIntegrator>();
 } // PA Mass test case
 
-TEST_CASE("PA Diffusion", "[PartialAssembly], [CUDA]")
+TEST_CASE("PA Diffusion", "[PartialAssembly], [GPU]")
 {
    test_pa_integrator<DiffusionIntegrator>();
 } // PA Diffusion test case
 
-TEST_CASE("PA Markers", "[PartialAssembly], [CUDA]")
+TEST_CASE("PA Markers", "[PartialAssembly], [GPU]")
 {
    const bool all_tests = launch_all_non_regression_tests;
    auto fname = GENERATE("../../data/star.mesh", "../../data/star-q3.mesh",
@@ -678,7 +749,7 @@ TEST_CASE("PA Markers", "[PartialAssembly], [CUDA]")
    REQUIRE(y_fa.Normlinf() == MFEM_Approx(0.0));
 }
 
-TEST_CASE("PA Boundary Mass", "[PartialAssembly], [CUDA]")
+TEST_CASE("PA Boundary Mass", "[PartialAssembly], [GPU]")
 {
    const bool all_tests = launch_all_non_regression_tests;
 
@@ -730,7 +801,48 @@ template <> struct ParTypeHelper<ParFiniteElementSpace>
 #endif
 }
 
-template <typename FES>
+template <typename CoeffType>
+std::unique_ptr<CoeffType> MakeCoeff(int);
+
+template <>
+std::unique_ptr<ConstantCoefficient> MakeCoeff<ConstantCoefficient>(int)
+{
+   return std::make_unique<ConstantCoefficient>(3.14159);
+}
+
+template <>
+std::unique_ptr<MatrixConstantCoefficient> MakeCoeff<MatrixConstantCoefficient>
+(int dim)
+{
+   DenseMatrix A(dim);
+   for (int i = 0; i < dim*dim; ++i)
+   {
+      A.GetData()[i] = 1.0 / (i + 3.0);
+   }
+   for (int i = 0; i < dim; ++i)
+   {
+      A(i,i) += 2.0 + i;
+   }
+   return std::make_unique<MatrixConstantCoefficient>(A);
+}
+
+template <> std::unique_ptr<SymmetricMatrixConstantCoefficient>
+MakeCoeff<SymmetricMatrixConstantCoefficient>(int dim)
+{
+   DenseSymmetricMatrix A(dim);
+   for (int i = 0; i < A.GetStoredSize(); ++i)
+   {
+      A.GetData()[i] = 1.0 / (i + 3.0);
+   }
+   for (int i = 0; i < dim; ++i)
+   {
+      A(i,i) += 2.0 + i;
+   }
+   return std::make_unique<SymmetricMatrixConstantCoefficient>(A);
+}
+
+template <typename CoeffType = ConstantCoefficient,
+          typename FES = FiniteElementSpace>
 void test_dg_diffusion(FES &fes)
 {
    using GF_t = typename ParTypeHelper<FES>::GF_t;
@@ -739,18 +851,20 @@ void test_dg_diffusion(FES &fes)
    GF_t x(&fes), y_fa(&fes), y_pa(&fes);
    x.Randomize(1);
 
-   ConstantCoefficient pi(3.14159);
+   const int dim = fes.GetMesh()->Dimension();
+   auto coeff = MakeCoeff<CoeffType>(dim);
 
    const real_t sigma = -1.0;
    const real_t kappa = 10.0;
 
    IntegrationRules irs(0, Quadrature1D::GaussLobatto);
-   const IntegrationRule &ir = irs.Get(fes.GetMesh()->GetFaceGeometry(0),
+   const IntegrationRule &ir = irs.Get(fes.GetMesh()->GetTypicalFaceGeometry(),
                                        2*fes.GetMaxElementOrder());
 
    BLF_t blf_fa(&fes);
-   blf_fa.AddInteriorFaceIntegrator(new DGDiffusionIntegrator(pi, sigma, kappa));
-   blf_fa.AddBdrFaceIntegrator(new DGDiffusionIntegrator(pi, sigma, kappa));
+   blf_fa.AddInteriorFaceIntegrator(
+      new DGDiffusionIntegrator(*coeff, sigma, kappa));
+   blf_fa.AddBdrFaceIntegrator(new DGDiffusionIntegrator(*coeff, sigma, kappa));
    (*blf_fa.GetFBFI())[0]->SetIntegrationRule(ir);
    (*blf_fa.GetBFBFI())[0]->SetIntegrationRule(ir);
    blf_fa.Assemble();
@@ -762,8 +876,9 @@ void test_dg_diffusion(FES &fes)
 
    BLF_t blf_pa(&fes);
    blf_pa.SetAssemblyLevel(AssemblyLevel::PARTIAL);
-   blf_pa.AddInteriorFaceIntegrator(new DGDiffusionIntegrator(pi, sigma, kappa));
-   blf_pa.AddBdrFaceIntegrator(new DGDiffusionIntegrator(pi, sigma, kappa));
+   blf_pa.AddInteriorFaceIntegrator(
+      new DGDiffusionIntegrator(*coeff, sigma, kappa));
+   blf_pa.AddBdrFaceIntegrator(new DGDiffusionIntegrator(*coeff, sigma, kappa));
    (*blf_pa.GetFBFI())[0]->SetIntegrationRule(ir);
    (*blf_pa.GetBFBFI())[0]->SetIntegrationRule(ir);
    blf_pa.Assemble();
@@ -792,7 +907,7 @@ std::vector<std::string> get_dg_test_meshes()
    return mesh_filenames;
 }
 
-TEST_CASE("PA DG Diffusion", "[PartialAssembly], [CUDA]")
+TEST_CASE("PA DG Diffusion", "[PartialAssembly], [GPU]")
 {
    const auto mesh_fname = GENERATE_COPY(from_range(get_dg_test_meshes()));
    const int order = GENERATE(1, 2);
@@ -804,12 +919,14 @@ TEST_CASE("PA DG Diffusion", "[PartialAssembly], [CUDA]")
    DG_FECollection fec(order, dim, BasisType::GaussLobatto);
    FiniteElementSpace fes(&mesh, &fec);
 
-   test_dg_diffusion(fes);
+   test_dg_diffusion<ConstantCoefficient>(fes);
+   test_dg_diffusion<MatrixConstantCoefficient>(fes);
+   test_dg_diffusion<SymmetricMatrixConstantCoefficient>(fes);
 }
 
 #ifdef MFEM_USE_MPI
 
-TEST_CASE("Parallel PA DG Diffusion", "[PartialAssembly][Parallel][CUDA]")
+TEST_CASE("Parallel PA DG Diffusion", "[PartialAssembly][Parallel][GPU]")
 {
    const auto mesh_fname = GENERATE_COPY(from_range(get_dg_test_meshes()));
    const int order = GENERATE(1, 2);
@@ -824,9 +941,40 @@ TEST_CASE("Parallel PA DG Diffusion", "[PartialAssembly][Parallel][CUDA]")
    DG_FECollection fec(order, dim, BasisType::GaussLobatto);
    ParFiniteElementSpace fes(&mesh, &fec);
 
-   test_dg_diffusion(fes);
+   test_dg_diffusion<ConstantCoefficient>(fes);
+   test_dg_diffusion<MatrixConstantCoefficient>(fes);
 }
 
 #endif
 
 } // namespace pa_kernels
+
+TEST_CASE("Dispatch Map Specializations")
+{
+   // The kernel specializations are registered the first time the associated
+   // object is created (in the constructor of a static local variable in the
+   // object's constructor). We create a dummy objects here to ensure that the
+   // kernels are registered before testing.
+
+   MassIntegrator{};
+   REQUIRE_FALSE(MassIntegrator::ApplyPAKernels::GetDispatchTable().empty());
+   REQUIRE_FALSE(MassIntegrator::DiagonalPAKernels::GetDispatchTable().empty());
+
+   DiffusionIntegrator{};
+   REQUIRE_FALSE(
+      DiffusionIntegrator::ApplyPAKernels::GetDispatchTable().empty());
+   REQUIRE_FALSE(
+      DiffusionIntegrator::DiagonalPAKernels::GetDispatchTable().empty());
+
+   Mesh mesh = Mesh::MakeCartesian2D(2, 2, Element::QUADRILATERAL);
+   H1_FECollection fec(1, mesh.Dimension());
+   FiniteElementSpace fes(&mesh, &fec);
+   fes.GetQuadratureInterpolator(IntRules.Get(mesh.GetElementGeometry(0), 1));
+
+   using QI = QuadratureInterpolator;
+   REQUIRE_FALSE(QI::TensorEvalKernels::GetDispatchTable().empty());
+   REQUIRE_FALSE(QI::GradKernels::GetDispatchTable().empty());
+   REQUIRE_FALSE(QI::DetKernels::GetDispatchTable().empty());
+   REQUIRE_FALSE(QI::EvalKernels::GetDispatchTable().empty());
+   REQUIRE_FALSE(QI::CollocatedGradKernels::GetDispatchTable().empty());
+}
