@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -11,9 +11,29 @@
 
 #include "mesh_test_utils.hpp"
 
+#include <numeric>
+
 namespace mfem
 {
 
+
+FiniteElementCollection *create_fec(FECType fectype, int p, int dim)
+{
+   switch (fectype)
+   {
+      case FECType::H1:
+         return new H1_FECollection(p, dim);
+      case FECType::ND:
+         return new ND_FECollection(p, dim);
+      case FECType::RT:
+         return new RT_FECollection(p - 1, dim);
+      case FECType::L2:
+         return new L2_FECollection(p, dim, BasisType::GaussLobatto);
+   }
+
+   return nullptr;
+}
+
 int CheckPoisson(Mesh &mesh, int order, int disabled_boundary_attribute)
 {
    constexpr int dim = 3;
@@ -68,6 +88,8 @@ int CheckPoisson(Mesh &mesh, int order, int disabled_boundary_attribute)
    bool satisfy_bc = true;
    Vector tvec;
    sol.GetTrueDofs(tvec);
+   ess_tdof_list.HostRead();
+   tvec.HostRead();
    for (auto dof : ess_tdof_list)
    {
       if (tvec[dof] != 0.0)
@@ -164,9 +186,10 @@ Mesh TetStarMesh()
    return mesh;
 }
 
-Mesh DividingPlaneMesh(bool tet_mesh, bool split)
+Mesh DividingPlaneMesh(bool tet_mesh, bool split, bool three_dim)
 {
-   auto mesh = Mesh("../../data/ref-cube.mesh");
+   auto mesh = three_dim ? Mesh("../../data/ref-cube.mesh") :
+               Mesh("../../data/ref-square.mesh");
    {
       Array<Refinement> refs;
       refs.Append(Refinement(0, Refinement::X));
@@ -405,6 +428,69 @@ Mesh CylinderMesh(Geometry::Type el_type, bool quadratic, int variant)
    return mesh;
 }
 
+void RefineSingleAttachedElement(Mesh &mesh, int vattr, int battr,
+                                 bool backwards)
+{
+   Array<Refinement> refs(1);
+   std::vector<int> ind(mesh.GetNBE());
+   if (backwards)
+   {
+      std::iota(ind.rbegin(), ind.rend(), 0);
+   }
+   else
+   {
+      std::iota(ind.begin(), ind.end(), 0);
+   }
+   for (int e : ind)
+   {
+      if (mesh.GetBdrAttribute(e) == battr)
+      {
+         int f, o, el1, el2;
+         mesh.GetBdrElementFace(e, &f, &o);
+         mesh.GetFaceElements(f, &el1, &el2);
+         if (mesh.GetAttribute(el1) == vattr)
+         { mesh.GeneralRefinement(Array<int> {el1}); return; }
+         if (mesh.GetAttribute(el2) == vattr)
+         { mesh.GeneralRefinement(Array<int> {el2}); return; }
+      }
+   }
+}
+
+void RefineSingleUnattachedElement(Mesh &mesh, int vattr, int battr,
+                                   bool backwards)
+{
+   std::set<int> attached_elements;
+   for (int e = 0; e < mesh.GetNBE(); e++)
+   {
+      if (mesh.GetBdrAttribute(e) == battr)
+      {
+         int f, o, el1, el2;
+         mesh.GetBdrElementFace(e, &f, &o);
+         mesh.GetFaceElements(f, &el1, &el2);
+         if (mesh.GetAttribute(el1) == vattr) { attached_elements.insert(el1); }
+         if (el2 >= 0 && mesh.GetAttribute(el2) == vattr) { attached_elements.insert(el2); }
+      }
+   }
+   if (backwards)
+   {
+      for (int i = mesh.GetNE() - 1; i >= 0; i--)
+         if (mesh.GetAttribute(i) == vattr && attached_elements.count(i) == 0)
+         {
+            mesh.GeneralRefinement(Array<int> {i});
+            return;
+         }
+   }
+   else
+   {
+      for (int i = 0; i < mesh.GetNE(); i++)
+         if (mesh.GetAttribute(i) == vattr && attached_elements.count(i) == 0)
+         {
+            mesh.GeneralRefinement(Array<int> {i});
+            return;
+         }
+   }
+}
+
 #ifdef MFEM_USE_MPI
 
 void TestVectorValueInVolume(Mesh &smesh, int nc_level, int skip, bool use_ND)
